
#version 3.8;

global_settings {
	assumed_gamma 1.8
	max_trace_level 15
	photons {
		count 2000
		autostop 0
		jitter 0.4
	}
}

#include "colors.inc"
#include "textures.inc"
#include "metals.inc"
#include "shapes.inc"
#include "pawn.inc"

// 
#macro atmos_media_params ()           
	intervals 5  //Set to 3 or 5 for preview; 8 or 10 for final rendering
	samples 1,40
	variance 1/256
	confidence 0.999  
#end

#macro cloud_media_params ()
	intervals 5 //Set to 5 or so for preview; 10 or 12 for final rendering
	samples 1,40
	variance 1/256
	confidence 0.999  
#end

// ******** CAMERA ********
camera {
	location <0,0.01,0> // found it easiest to work from near the origin
	direction <0,0,1>
	right <2,0,0>
	rotate <-10,0,0>
	angle 80
}

// ******** LIGHTS ********

// used to give the landscape a little better ilumination since the main light angle is so low
light_source {
	<1000,10000,10000>
	rgb <0.5,0.5,1>*1
	media_interaction off
}

// light for the sun and the object used to represent the sun
light_source {
	<0,0,9000000>
	rgb 2
	rotate <-5,0,0>
	rotate <0,20,0>
	media_interaction on
}

disc { <0,0,0>, <0,0,-1>, 30
	pigment {
		spherical
		poly_wave 3
		color_map {
			[ 0.0 rgb 0 ]
			[ 0.9 rgb 15 ]
		}
	}
	scale 200000
	translate <0,0,1000000>
	rotate <-5,0,0>
	rotate <0,20,0>
	no_shadow
}

// BLUE SKY
#declare BlueSky = 
media {
	scattering {
		4,
		rgb <0.1,0.15,0.3>*0.2
		extinction 1.1
	}
	density {
		planar
		poly_wave 2
		color_map {
			[ 0 rgb 0 ]
			[ 1 rgb 1 ]
		}
		scale 20
	}
	atmos_media_params() 
}
// comment out the next line to turn off blue sky
sphere { <0,0,0>, 20
	hollow
	pigment { rgbf 1 }
	interior {
		media { BlueSky }
	}
} 

//RED GLOW
#declare RedGlow =
media {
	scattering {
		5,
		rgb <1,0.8,0>*0.05
		extinction 0
		eccentricity 0.5
	}
	absorption rgb <0.05,0.7,0.95>*0.2
	density {
		planar
		poly_wave 0.7
		color_map  {
			[ 0 rgb 0 ]
			[ 1 rgb 1 ]
		}
		scale <20,1,20>
	}
	atmos_media_params()
}
// comment out the next line to turn off red glow
sphere { <0,0,0>, 20
	hollow
	pigment { rgbf 1 }
	scale <1,0.8/20,1>
	interior {
		media { RedGlow }
	}
}

//GROUND HAZE
#declare GroundHaze =
media {
	absorption rgb <0.5,0.7,1>*0.5 
	density {
		planar
		color_map {
			[ 0.35 rgb 0 ]
			[ 0.50 rgb 1 ]
		}
		scale <1,2,1>
		translate <0,-1,0>
	}
	atmos_media_params()
}
// comment out the next line to turn off ground haze
difference {
	cylinder { <0,-1,0>, <0,1,0>, 20 }
	cylinder { <0,-1.1,0>, <0,1.1,0>, 2 }
	hollow
	pigment { rgbf 1 }
	interior {
		media { GroundHaze }
	}
}

//CLOUD PATTERN 
//Getting this right is a whole project in itself!
#macro CloudPat ()
	granite
	color_map {
		[ 0.5000 rgb 0 ]
		[ 0.5001 rgb 1 ]
	}
	scale 5
	scale 7
	warp {
		turbulence 0.03
		lambda 1.4
		omega 2
		octaves 5
	}
	scale 1/7 
	scale 1
	translate <0,0,3>
	translate <-0.6,0,0>
	scale <2,0.5,2>
#end

#declare Clouds =
// the container object is highly dependent on your own preference
sphere { <0,0,0>, 2
	scale <4,2/40,4>
	translate <0,0.75,0>
	hollow 
	pigment { rgbf 1 } 
	interior {
		media {
			scattering {
				1,
				rgb 6
				extinction 1
			}
			emission rgb <0.6,0.5,0.9>*1 // used to try to lighten the color of the clouds
			density {
				CloudPat()
			}
			cloud_media_params()
		}
	}
	translate <0,-0.75,0>
	rotate <-4,0,0>
	translate <0,0.75,0> // attempt to tilt twds sun for better illumination
}

object { Clouds } // comment out this line to turn off the clouds

//------!---------!---------!---------!---------!---------!---------!---------!
union {
	#declare X=-8;
	#while (X<8)
		#declare Z=-8;
		#while (Z<8)
			object {
				Round_Box_Union(<-1,-1,-1>, <0,0,0>, 0.00625)
				#if (even(X+Z))
					pigment { rgb 0 }
				#else
					pigment { rgb 1 }
				#end
				finish {
					reflection { 0.1, 1.0 fresnel on }
	                specular 0.8
	                roughness 0.02
	                metallic 0.75
	                conserve_energy
                }
                normal {
					//bumps 0.1
	                //scale 0.00005
					bumps 0.01
	                scale 0.000005
                }
				interior {
					ior 1.5
				}
				translate <X,0,Z>
			}
			#declare Z=Z+1;
		#end
		#declare X=X+1;
	#end
	scale 0.2
	translate <-0.015,0,0>
}

//------!---------!---------!---------!---------!---------!---------!---------!
// Pawn object by Douglas Otwell

// Felt by Ville Saari
#declare FeltBottom =
texture {
	pigment {
		agate
		agate_turb 1.5
		scale 0.33
		color_map {
			[ 0.11 srgbf <0.01,0.8,0.2,1> ]
			[ 0.33 srgb <0.01,0.8,0.2> ]
		}
	}
	normal {
		agate 0.33
		agate_turb 1.5
		scale 0.33
	}
	finish {
		//ambient 0.01
		emission 0.03
		diffuse 0.3
		specular 0.05
		roughness 0.2
		reflection {
			0,0.05
		}
		conserve_energy
		subsurface {
			translucency <0.025,0.4,0.2>
		}
	}
}

#declare DO_Pawn =
object {
	merge {

		difference {
			intersection  {
				object { Cylinder_Y }
				plane { <0,1,0>, -1 inverse }
				plane { <0,1,0>,  1 }
				scale <8,12.7468,8>
			}
			quartic {
				<1,0,0,0,2,0,0,2,0,-738,0,0,0,0,0,0,0,0,0,0,1,0,0,2,0,162,0,0,0,0,1,0,-738,0,6561>
			}
		}

		quartic {
			<1,0,0,0,2,0,0,2,0,-132.5,0,0,0,0,0,0,0,0,0,0,1,0,0,2,0,123.5,0,0,0,0,1,0,-132.5,0,3813.0625>
			translate <0,-11.2468,0>
		}

		quartic{
			<1,0,0,0,2,0,0,2,0,-132.5,0,0,0,0,0,0,0,0,0,0,1,0,0,2,0,123.5,0,0,0,0,1,0,-132.5,0,3813.0625>
			translate <0,11.2468,0>
		}

		//	Base
		intersection {
			intersection {
				object { Cylinder_Y }
				plane { <0,1,0>, -1 inverse }
				plane { <0,1,0>,  1 }
				scale <12,3,12>
				translate <0,-15.7468,0>
			}
			object { QCone_Y translate <0,-2,0> }
		}

		// Felt pad
		intersection {
			object { Cylinder_Y }
			plane { <0,1,0>, -1 inverse }
			plane { <0,1,0>,  1 }
			scale <12,0.5,12>
			translate <0,-19.2468,0>
			texture { FeltBottom }
		}

		//disc { <0,0,0>, <0,1,0>, 11.8
		//	pigment { rgb <0,0,0> }
		//	translate <0,-17.7468,0>
		//}

		//	Ball on top
		sphere { <0,17.7468,0>, 7 }
	}
	
	//bounded_by {
	//	intersection  {
	//		object { Cylinder_Y }
	//		plane { <0,1,0>, -1 inverse }
	//		plane { <0,1,0>, 1 }
	//		scale <14,26,14>
	//	}
	//}
	
	translate <0,17.7468,0>
	scale 0.06
}
//------!---------!---------!---------!---------!---------!---------!---------!

object {
	DO_Pawn
	texture {
		pigment { rgb 0 }
		finish { Metal }
	}
    scale 0.1
	translate <-0.15,0.011,0.55>
	photons {  //not sure this does anything!
		target
		reflection on
		refraction on
	}
}

//------!---------!---------!---------!---------!---------!---------!---------!
// Gilles Tran glass texture
#declare F_Glass =
finish {
	ambient 0
	diffuse 0
	specular 0.8
	roughness 0.0003
	phong 1 
	phong_size 400
	reflection {
		0.01, 1
		fresnel on
	}
	conserve_energy
	irid { 0.9 turbulence 0.2 }
}
#declare T_Glass =
texture {
	pigment { rgbf <1, 1, 1, 0.95> }
	finish { F_Glass }
} 
#declare I_Glass =
interior{
	ior 1.51
	fade_power 1001
	fade_distance 0.2
	fade_color <0.995,0.995,1>
	dispersion 1.001
	dispersion_samples 10
	caustics 1
}

//------!---------!---------!---------!---------!---------!---------!---------!
// Richard Dault cracked sphere
// https://news.povray.org/povray.binaries.images/thread/%3C39be4b10%40news.povray.org%3E/?ttop=441819&toff=11870
#declare Radius = 0.02;
//#declare ball = function { x*x+y*y+z*z-Radius*Radius }
#declare ball = function { f_sphere(x, y, z, Radius) }
#declare crack1 =
function {
	pigment {
		marble
		turbulence 0.7
		color_map {
			[ 0.000 rgb 1 ]
			[ 0.500 rgb 1 ]
			[ 0.500 rgb 0 ]
			[ 0.505 rgb 0 ]
			[ 0.505 rgb 1 ]
			[ 1.000 rgb 1 ]
		}
		scale 4
	}
}
#declare crack2 =
function {
	pigment {
		marble
		turbulence 0.7
		color_map {  //inverse colors of "crack1"
			[ 0.000 rgb 0 ]
			[ 0.500 rgb 0 ]
			[ 0.500 rgb 1 ]
			[ 0.505 rgb 1 ]
			[ 0.505 rgb 0 ]
			[ 1.000 rgb 0 ]
		}
		scale 4
	}
}

#declare CrackOne =
isosurface {
	function {
		//ball(x,y,z)-crack1(x,y,z)
		ball(x,y,z)-crack1(x,y,z).x
	}
	max_trace 3
	//method 2
	accuracy 0.001
	threshold 0
	max_gradient 20
	contained_by {
		sphere { <0,0,0>, 1.3 }
	}
}
#declare CrackTwo =  //suggested by Mike Williams
isosurface {
	function {
		//ball(x,y,z)+crack2(x,y,z)*6-1
		ball(x,y,z)+crack2(x,y,z).x*6-1
	}
	max_trace 3
	//method 2
	accuracy 0.001
	threshold 0
	max_gradient 20
	contained_by {
		sphere { <0,0,0>, 2.3 }
	}
}

//------!---------!---------!---------!---------!---------!---------!---------!
object {
	intersection {
		object { CrackTwo }
		sphere { <0,0,0>, Radius*0.95 inverse }
		material {
			texture { T_Glass }
			interior { I_Glass }
		}
		photons {
			target 1.0
			refraction on
			reflection off
			collect off
		}
	}
	scale 0.02
	translate <0.075,0.02,0.1>
}
//------!---------!---------!---------!---------!---------!---------!---------!

